/*  Linux word  The Japanese word processor
 *  initwindow.c
 *
 *  Copyright (C) 2000 YOSHIDA Daisuke
 *  Copyright (C) 1999, 2000 MITUIWA Yukio, NAKATANI Chie
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

#include "config.h"

#include "initwindow.h"
#include "word.h"
#include "gtkwrap.h"
#include "createhelp.h"
#include "createsetwin.h"
#include "fileio.h"
#include "edit.h"

#include "icons/open.xpm"
#include "icons/save.xpm"
#include "icons/inserttext.xpm"
#include "icons/readtext.xpm"
#include "icons/print.xpm"
#include "icons/texout.xpm"
#include "icons/cut.xpm"
#include "icons/copy.xpm"
#include "icons/paste.xpm"
#include "icons/nextpage.xpm"
#include "icons/prevpage.xpm"
#include "icons/alignright.xpm"
#include "icons/aligncenter.xpm"
#include "icons/alignleft.xpm"
#include "icons/help.xpm"
#include "icons/uline.xpm"
#include "icons/bold.xpm"
#include "icons/italic.xpm"

/*****************************************
 * ᥤ󥦥ɥ
 *****************************************/
void init_mainwindow (gchar *filename)
{
  GtkWidget *work, *window;
  GtkWidget *vbox;
  GtkWidget *menu_bar;
  GtkWidget *file_menu, *file_menu_exportsub, *file_menu_importsub;
  static GtkWidget *edit_menu;
  GtkWidget *edit_menu_editsub1, *edit_menu_editsub2;
  GtkWidget *help_menu;
  GtkWidget *set_menu;
  GtkWidget *tol_bar;
  GtkWidget *ctl_bar;
  GtkAccelGroup *accel_group;
  GtkWidget *menuitems;

  GList *cbitems;
  gint i;
  gchar rcname[128];
  static gchar points[][4] = {"7","8","9","10","11","12","13","14","15","16",
			      "17","18","20","24","26","28","30","32","34",
			      "36","38","40","44","48","52","56","60","64",
			      "68","72","80","88","96","102","108","116",
			      "127","0"};

  /* ɥ */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy",
		     GTK_SIGNAL_FUNC(word_main_quit),NULL);
  /* 졼 */
  accel_group = gtk_accel_group_new();
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  /* .wordfontrcѡƻѲǽʥե̾ */
  strcpy(rcname, (gchar*)getenv("HOME"));
  strcat(rcname, "/.wordfontrc");
  if (!read_fontrc("./wordfontrc"))
    if(!read_fontrc(rcname))
      read_fontrc("/etc/wordfontrc");

  /* ᥤvboxդ */
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  gtk_widget_show(vbox);

  /* vbox˥˥塼Сդ */
  menu_bar = gtk_menu_bar_new();
  box_pack(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);

  /* ֥ե */
  file_menu = gtk_menu_new();
  menuitems =
    set_menu_item(file_menu, "...", GTK_SIGNAL_FUNC(read_word));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_o, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  menuitems =
    set_menu_item(file_menu, "̾Ĥ¸...",
		GTK_SIGNAL_FUNC(save_word));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_s, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
#ifdef PSVIEWER
  menuitems =
    set_menu_item(file_menu, "ץӥ塼...", GTK_SIGNAL_FUNC(preview_out));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_v, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
#endif
#ifdef LPRCOMMAND
  menuitems =
    set_menu_item(file_menu, "...", GTK_SIGNAL_FUNC(lpr_out));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_p, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
#endif
  set_menu_separate(file_menu);
  file_menu_importsub = gtk_menu_new();
  set_menu_item(file_menu_importsub, "ƥɤ߹...",
		GTK_SIGNAL_FUNC(read_text));
  set_menu_item(file_menu_importsub, "ƥ...",
		GTK_SIGNAL_FUNC(insert_textfile));
  set_sub_menu(file_menu_importsub, file_menu, "ݡ");
  file_menu_exportsub = gtk_menu_new();
  set_menu_item(file_menu_exportsub, "ƥ...", GTK_SIGNAL_FUNC(plaintext_out));
  set_menu_item(file_menu_exportsub, "PostScript...", GTK_SIGNAL_FUNC(print_word));
  set_menu_item(file_menu_exportsub, "TeX...", GTK_SIGNAL_FUNC(tex_out));
  set_sub_menu(file_menu_exportsub, file_menu, "ݡ");
  set_menu_separate(file_menu);
  menuitems =
    set_menu_item(file_menu, "λ", GTK_SIGNAL_FUNC(word_main_quit));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_x, GDK_MOD1_MASK, GTK_ACCEL_VISIBLE);
  set_main_menu(file_menu, menu_bar, "ե");

  /* Խ */
  edit_menu = gtk_menu_new();
  menuitems =
    set_menu_item(edit_menu, "ڤ", GTK_SIGNAL_FUNC(delete_rev));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_x, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  menuitems =
    set_menu_item(edit_menu, "ԡ", GTK_SIGNAL_FUNC(copy_rev));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_c, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  menuitems =
    set_menu_item(edit_menu, "Žդ", GTK_SIGNAL_FUNC(paste_master));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_v, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  set_menu_separate(edit_menu);
  edit_menu_editsub1 = gtk_menu_new();
  menuitems =
    set_menu_item(edit_menu_editsub1, "Ǥ", GTK_SIGNAL_FUNC(prev_page));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_Page_Up, 0, GTK_ACCEL_VISIBLE);
  menuitems =
    set_menu_item(edit_menu_editsub1, "Ǥ", GTK_SIGNAL_FUNC(next_page));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_Page_Down, 0, GTK_ACCEL_VISIBLE);
  set_menu_separate(edit_menu_editsub1);
  menuitems =
    set_menu_item(edit_menu_editsub1, "ƬǤ", GTK_SIGNAL_FUNC(top_page));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_Home, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  menuitems =
    set_menu_item(edit_menu_editsub1, "ǽǤ", GTK_SIGNAL_FUNC(end_page));
  gtk_widget_add_accelerator (menuitems, "activate", accel_group,
			      GDK_End, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  set_menu_separate(edit_menu_editsub1);
  set_menu_item(edit_menu_editsub1, "ֹ", GTK_SIGNAL_FUNC(jump_page));
  set_sub_menu(edit_menu_editsub1, edit_menu, "Ǥư");
  edit_menu_editsub2 = gtk_menu_new();
  set_menu_item(edit_menu_editsub2, "", GTK_SIGNAL_FUNC(set_center_aline));
  set_menu_item(edit_menu_editsub2, "", GTK_SIGNAL_FUNC(set_left_aline));
  set_menu_item(edit_menu_editsub2, "", GTK_SIGNAL_FUNC(set_right_aline));
  set_sub_menu(edit_menu_editsub2, edit_menu, "ʸ·");
  set_main_menu(edit_menu, menu_bar, "Խ");

  /*  */
  set_menu = gtk_menu_new();
  set_menu_item(set_menu, "Դ֤", GTK_SIGNAL_FUNC(set_vspace));
  set_menu_separate(set_menu);
  set_menu_item(set_menu, "ѻ", GTK_SIGNAL_FUNC(set_margin));
/*  set_menu_item(set_menu, "Ȥ", GTK_SIGNAL_FUNC(set_tabular));*/
  set_main_menu(set_menu, menu_bar, "");

  /* ֥إס */
  help_menu = gtk_menu_new();
  set_menu_item(help_menu, "ޥ˥奢", GTK_SIGNAL_FUNC(on_man_activate));
  work = set_main_menu(help_menu, menu_bar, "إ");
  gtk_menu_item_right_justify(GTK_MENU_ITEM(work));

  /* vbox˥ġСդ */
  tol_bar = create_toolbar (window);
  box_pack(GTK_BOX(vbox), tol_bar, FALSE, FALSE, 0);

  /* vbox˥ȥСդ */
  ctl_bar = gtk_hbox_new(FALSE, 0);
  box_pack(GTK_BOX(vbox), ctl_bar, FALSE, FALSE, 0);

  /* ե򥳥ܥܥå */
  cbitems = NULL;
  for(i = 0;fontnames[KANJI_FONT][i][0] != '0';i++)
    cbitems = g_list_append(cbitems, fontnames[KANJI_FONT][i]);
  gtk_combo_set_popdown_strings(GTK_COMBO((CUR_KANJI = gtk_combo_new())),
				cbitems);
  gtk_entry_set_editable(KANJI_ENTRY, FALSE);
  gtk_signal_connect(GTK_OBJECT(KANJI_ENTRY), "changed",
                     (GtkSignalFunc)change_word_attr, (gpointer)CHANGE_KANJI);
  box_pack(GTK_BOX(ctl_bar), CUR_KANJI, FALSE, TRUE, 4);
  g_list_free(cbitems);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(CUR_KANJI)->entry),
                     "leave_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(CUR_KANJI)->popwin),
                     "leave_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);

  /* ʸե򥳥ܥܥå */
  cbitems = NULL;
  for(i = 0;fontnames[ASCII_FONT][i][0] != '0';i++)
    cbitems = g_list_append(cbitems, fontnames[ASCII_FONT][i]);
  gtk_combo_set_popdown_strings(GTK_COMBO((CUR_ASCII = gtk_combo_new())),
				cbitems);
  gtk_entry_set_editable(ASCII_ENTRY, FALSE);
  gtk_signal_connect(GTK_OBJECT(ASCII_ENTRY), "changed",
                     (GtkSignalFunc)change_word_attr, (gpointer)CHANGE_ASCII);
  box_pack(GTK_BOX(ctl_bar), CUR_ASCII, FALSE, TRUE, 4);
  g_list_free(cbitems);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(CUR_ASCII)->entry),
                     "leave_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(CUR_ASCII)->popwin),
                     "leave_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);

  /* եȤΥݥ򥳥ܥܥå */
  cbitems = NULL;
  for(i = 0;points[i][0] != '0';i++)
    cbitems = g_list_append(cbitems, points[i]);
  gtk_combo_set_popdown_strings(GTK_COMBO((cur_point = gtk_combo_new())),
				cbitems);
  gtk_entry_set_editable(POINT_ENTRY, FALSE);
  gtk_entry_set_text(POINT_ENTRY, "14");
  gtk_widget_set(GTK_WIDGET(POINT_ENTRY), "width", 35, NULL);
  gtk_signal_connect(GTK_OBJECT(POINT_ENTRY), "changed",
                     (GtkSignalFunc)change_word_attr, (gpointer)CHANGE_POINT);
  box_pack(GTK_BOX(ctl_bar), cur_point, FALSE, TRUE, 4);
  g_list_free(cbitems);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(cur_point)->entry),
                     "leave_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(cur_point)->popwin),
                     "leave_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);

  /* ܥ */
  uline = create_xpmtoggle (window, uline_xpm);
  gtk_signal_connect(GTK_OBJECT(uline), "button_press_event", 
                     (GtkSignalFunc)toggle_button_press,
		     (gpointer)CHANGE_ULINE);
  box_pack(GTK_BOX(ctl_bar), uline, FALSE, TRUE, 4);

  /* ܥ */
  bold = create_xpmtoggle (window, bold_xpm);
  gtk_signal_connect(GTK_OBJECT(bold), "button_press_event", 
                     (GtkSignalFunc)toggle_button_press,
		     (gpointer)CHANGE_BOLD);
  box_pack(GTK_BOX(ctl_bar), bold, FALSE, TRUE, 4);

  /* Υܥ */
  italic = create_xpmtoggle (window, italic_xpm);
  gtk_signal_connect(GTK_OBJECT(italic), "button_press_event", 
                     (GtkSignalFunc)toggle_button_press,
		     (gpointer)CHANGE_ITALIC);
  box_pack(GTK_BOX(ctl_bar), italic, FALSE, TRUE, 4);

  /* vboxʸβѥ֥ȥåפդ */
  tab_set = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(tab_set), -1, 16);
  gtk_signal_connect(GTK_OBJECT(tab_set), "expose_event",
		     (GtkSignalFunc)replace_tab, NULL);
  gtk_signal_connect(GTK_OBJECT(tab_set), "configure_event",
		     (GtkSignalFunc)replace_tab, NULL);
  gtk_signal_connect(GTK_OBJECT(tab_set), "motion_notify_event",
		     (GtkSignalFunc)tab_motion_notify, NULL);
  gtk_signal_connect(GTK_OBJECT(tab_set), "button_press_event",
		     (GtkSignalFunc)tab_button_event, NULL);
  gtk_signal_connect(GTK_OBJECT(tab_set), "button_release_event",
		     (GtkSignalFunc)tab_button_event, NULL);
  gtk_widget_set_events(tab_set, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK |
			GDK_KEY_PRESS_MASK | GDK_POINTER_MOTION_MASK |
			GDK_POINTER_MOTION_HINT_MASK |
			GDK_BUTTON_RELEASE_MASK);
  box_pack(GTK_BOX(vbox), tab_set, FALSE, FALSE, 0);

  /* ʸΰ */
  init_document_area();
  tab_draw(tab_set, page_x, TAB_MAX);

  /* vbox˥դ */
  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  box_pack(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

  /* ʸΰդ */
  etext = gtk_drawing_area_new();
  gtk_drawing_area_size (GTK_DRAWING_AREA (etext), page_x, page_y);
  gtk_signal_connect(GTK_OBJECT(etext), "expose_event",
		     (GtkSignalFunc) expose_event, NULL);
  gtk_signal_connect(GTK_OBJECT(etext), "configure_event",
		     (GtkSignalFunc) configure_event, NULL);
  gtk_signal_connect(GTK_OBJECT(etext), "motion_notify_event",
		     (GtkSignalFunc) motion_notify_event, NULL);
  gtk_signal_connect(GTK_OBJECT(etext), "button_press_event",
		     (GtkSignalFunc) mouse_button_3, edit_menu);
  gtk_signal_connect(GTK_OBJECT(etext), "button_release_event",
		     (GtkSignalFunc) button_press_event, NULL);
  gtk_signal_connect(GTK_OBJECT(etext), "key_press_event",
		     (GtkSignalFunc) key_press_event, NULL);
  gtk_signal_connect(GTK_OBJECT(etext), "leave_notify_event",
		     (GtkSignalFunc) leave_notify_event, NULL);
  gtk_signal_connect(GTK_OBJECT(etext), "enter_notify_event",
		     (GtkSignalFunc) enter_notify_event, NULL);
  gtk_widget_set_events(etext, GDK_EXPOSURE_MASK | GDK_LEAVE_NOTIFY_MASK |
			GDK_ENTER_NOTIFY_MASK | GDK_BUTTON_PRESS_MASK |
			GDK_KEY_PRESS_MASK | GDK_POINTER_MOTION_MASK | 
                        GDK_POINTER_MOTION_HINT_MASK |
			GDK_BUTTON_RELEASE_MASK);
  gtk_widget_set_extension_events(etext, GDK_EXTENSION_EVENTS_ALL);
  GTK_WIDGET_SET_FLAGS (etext, GTK_CAN_FOCUS);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(scrolled_window),
					 etext);
  gtk_widget_show (etext);
  gtk_widget_grab_focus(etext);

  gtk_widget_set_usize (window, 580, 400);
  gtk_widget_show (window);
  if (filename != NULL)
    read_document_from_name (filename);
}

/*****************************************
 * .xfontrcѡ
 *****************************************/
int read_fontrc(gchar *name) { /* ϥե̾ */
  FILE *fp;
  int i, a;

  /*
    fontnamesɽե̾
    xfontnamesXϤե̾
    psfontnamesPSե˽񤭹ե̾
    */

  if((fp = fopen(name, "r")) != NULL) {
    i = 0;
    for(a = 0;(fontnames[ASCII_FONT][i][a] = fgetc(fp)) != '\n';a++)
      if(fontnames[ASCII_FONT][i][a] == ' ') a--;
    fontnames[ASCII_FONT][i][a] = 0;
    fgets(xfontnames[ASCII_FONT][i], 128, fp);
    xfontnames[ASCII_FONT][i][strlen(xfontnames[ASCII_FONT][i]) - 1] = '\0';
    fgets(psfontnames[ASCII_FONT][i], 128, fp);
    psfontnames[ASCII_FONT][i][strlen(psfontnames[ASCII_FONT][i]) - 1] = '\0';
    while(!feof(fp) && strcmp(fontnames[ASCII_FONT][i], "-end-font-def-")) {
      i++;
      for(a = 0;(fontnames[ASCII_FONT][i][a] = fgetc(fp)) != '\n';a++)
        if(fontnames[ASCII_FONT][i][a] == ' ') a--;
      fontnames[ASCII_FONT][i][a] = 0;
      fgets(xfontnames[ASCII_FONT][i],128,fp);
      xfontnames[ASCII_FONT][i][strlen(xfontnames[ASCII_FONT][i]) - 1] = '\0';
      fgets(psfontnames[ASCII_FONT][i],128,fp);
      psfontnames[ASCII_FONT][i][strlen(psfontnames[ASCII_FONT][i]) - 1] = '\0';
    }
    strcpy(fontnames[ASCII_FONT][i],"0");
    strcpy(xfontnames[ASCII_FONT][i],"0");
    strcpy(psfontnames[ASCII_FONT][i],"0");
    i = 0;
    for(a = 0;(fontnames[KANJI_FONT][i][a] = fgetc(fp)) != '\n';a++)
      if(fontnames[KANJI_FONT][i][a] == ' ') a--;
    fontnames[KANJI_FONT][i][a] = 0;
    fgets(xfontnames[KANJI_FONT][i],128,fp);
    xfontnames[KANJI_FONT][i][strlen(xfontnames[KANJI_FONT][i]) - 1] = '\0';
    fgets(psfontnames[KANJI_FONT][i],128,fp);
    psfontnames[KANJI_FONT][i][strlen(psfontnames[KANJI_FONT][i]) - 1] = '\0';
    while(!feof(fp) && strcmp(fontnames[KANJI_FONT][i], "-end-font-def-")) {
      i++;
      for(a = 0;(fontnames[KANJI_FONT][i][a] = fgetc(fp)) != '\n';a++)
        if(fontnames[KANJI_FONT][i][a] == ' ') a--;
      fontnames[KANJI_FONT][i][a] = 0;
      fgets(xfontnames[KANJI_FONT][i],128,fp);
      xfontnames[KANJI_FONT][i][strlen(xfontnames[KANJI_FONT][i]) - 1] = '\0';
      fgets(psfontnames[KANJI_FONT][i],128,fp);
      psfontnames[KANJI_FONT][i][strlen(psfontnames[KANJI_FONT][i]) - 1] = '\0';
    }
    strcpy(fontnames[KANJI_FONT][i],"0");
    strcpy(xfontnames[KANJI_FONT][i],"0");
    strcpy(psfontnames[KANJI_FONT][i],"0");
    fclose(fp);
    return TRUE;
  }
  strcpy(fontnames[ASCII_FONT][0],"system");
  strcpy(xfontnames[ASCII_FONT][0],"-*-*-*-");
  strcpy(psfontnames[ASCII_FONT][0],"/Times-Roman");
  strcpy(fontnames[ASCII_FONT][1],"0");
  strcpy(xfontnames[ASCII_FONT][1],"0");
  strcpy(psfontnames[ASCII_FONT][1],"0");
  strcpy(fontnames[KANJI_FONT][0],"system");
  strcpy(xfontnames[KANJI_FONT][0],"-*-*-*-");
  strcpy(psfontnames[KANJI_FONT][0],"/Ryumin-Light");
  strcpy(fontnames[KANJI_FONT][1],"0");
  strcpy(xfontnames[KANJI_FONT][1],"0");
  strcpy(psfontnames[KANJI_FONT][1],"0");
  return FALSE;
}

/*****************************************
 * ġС
 *****************************************/
GtkWidget* create_toolbar (GtkWidget *parent_win)
{
  GtkWidget *hbox1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkWidget *button4;
  GtkWidget *button5;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  hbox1 = gtk_hbox_new (FALSE, 2);
  gtk_container_border_width (GTK_CONTAINER (hbox1), 2);

  button1 = create_xpmbutton (parent_win, open_xpm);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button1, "", NULL);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (read_word),
                      NULL);

  button2 = create_xpmbutton (parent_win, save_xpm);
  gtk_widget_show (button2);
  gtk_box_pack_start (GTK_BOX (hbox1), button2, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button2, "̾դ¸", NULL);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (save_word),
                      NULL);

  button1 = create_xpmbutton (parent_win, readtext_xpm);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button1, "ƥȤɤ߹", NULL);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (read_text),
                      NULL);

  button1 = create_xpmbutton (parent_win, inserttext_xpm);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button1, "ƥȤ", NULL);
  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (insert_textfile),
                      NULL);

  button3 = create_xpmbutton (parent_win, print_xpm);
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (hbox1), button3, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button3, "", NULL);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (print_word),
                      NULL);

  button4 = create_xpmbutton (parent_win, texout_xpm);
  gtk_widget_show (button4);
  gtk_box_pack_start (GTK_BOX (hbox1), button4, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button4, "TeXǽ", NULL);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked",
                      GTK_SIGNAL_FUNC (tex_out),
                      NULL);

  button5 = create_xpmbutton (parent_win, cut_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "ڤ", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (delete_rev),
                      NULL);

  button5 = create_xpmbutton (parent_win, copy_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "ԡ", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (copy_rev),
                      NULL);

  button5 = create_xpmbutton (parent_win, paste_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "Žդ", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (paste_master),
                      NULL);

  button5 = create_xpmbutton (parent_win, nextpage_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (next_page),
                      NULL);

  button5 = create_xpmbutton (parent_win, prevpage_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (prev_page),
                      NULL);

  button5 = create_xpmbutton (parent_win, alignleft_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "·", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (set_left_aline),
                      NULL);

  button5 = create_xpmbutton (parent_win, aligncenter_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "·", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (set_center_aline),
                      NULL);

  button5 = create_xpmbutton (parent_win, alignright_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "·", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (set_right_aline),
                      NULL);

  button5 = create_xpmbutton (parent_win, help_xpm);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox1), button5, FALSE, FALSE, 0);
  gtk_tooltips_set_tip (tooltips, button5, "إ", NULL);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (on_man_activate),
                      NULL);

  return hbox1;
}

/*****************************************
 * ܥ󤬲줿饯å˥塼طҤ
 *****************************************/
gboolean mouse_button_3 
(GtkWidget *widget, GdkEventButton *event, GtkMenu *popup) 
{
  if (event->type == GDK_BUTTON_PRESS) {
    if (event->button == 3) {
      /* å˥塼ݥåץå */
      gtk_menu_popup (GTK_MENU(popup), NULL, NULL, NULL, NULL,
		      event->button, event->time);
      return TRUE;
    }
  }
  return button_press_event (widget, event);
}
